use strict;
use warnings;

# this test was generated with Dist::Zilla::Plugin::Test::EOL 0.18

use Test::More 0.88;
use Test::EOL;

my @files = (
    'lib/Params/CheckCompiler.pm',
    'lib/Params/CheckCompiler/Compiler.pm',
    'lib/Params/CheckCompiler/Exception/BadArguments.pm',
    'lib/Params/CheckCompiler/Exception/Extra.pm',
    'lib/Params/CheckCompiler/Exception/Required.pm',
    'lib/Params/CheckCompiler/Exception/ValidationFailedForMooseTypeConstraint.pm',
    't/00-report-prereqs.dd',
    't/00-report-prereqs.t',
    't/args-check.t',
    't/default.t',
    't/extra.t',
    't/moose.t',
    't/required.t',
    't/specio.t',
    't/type-tiny.t'
);

eol_unix_ok($_, { trailing_whitespace => 1 }) foreach @files;
done_testing;
