package Params::CheckCompiler::Exception::BadArguments;

use strict;
use warnings;

our $VERSION = '0.01';

use Moo;

extends 'Throwable::Error';

1;

# ABSTRACT: Exception thrown when @_ does not contain a hash or hashref

__END__

=pod

=encoding UTF-8

=head1 NAME

Params::CheckCompiler::Exception::BadArguments - Exception thrown when @_ does not contain a hash or hashref

=head1 VERSION

version 0.01

=head1 SUPPORT

Bugs may be submitted through L<the RT bug tracker|http://rt.cpan.org/Public/Dist/Display.html?Name=Params-CheckCompiler>
(or L<bug-params-checkcompiler@rt.cpan.org|mailto:bug-params-checkcompiler@rt.cpan.org>).

I am also usually active on IRC as 'drolsky' on C<irc://irc.perl.org>.

=head1 AUTHOR

Dave Rolsky <autarch@urth.org>

=head1 COPYRIGHT AND LICENCE

This software is Copyright (c) 2016 by Dave Rolsky.

This is free software, licensed under:

  The Artistic License 2.0 (GPL Compatible)

=cut
