#!perl

# This file was automatically generated by Dist::Zilla::Plugin::Test::TidyAll v0.02

use Test::Code::TidyAll 0.24;
use Test::More 0.88;

tidyall_ok(
     verbose => ( exists $ENV{TEST_TIDYALL_VERBOSE} ? $ENV{TEST_TIDYALL_VERBOSE} : 1 ),
);

done_testing();
